"use strict";
/**
 * link-local-dependencies
 * Install local lerna dependencies
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.linkAllLocalDependencies = exports.linkLocalDependencies = exports.linkLocalDependenciesIn = void 0;
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const trace_1 = require("@strong-roots-capital/trace");
const io_ts_1 = require("@typescript-tools/io-ts");
const package_manifests_1 = require("@typescript-tools/package-manifests");
const debug_1 = __importDefault(require("debug"));
const E = __importStar(require("fp-ts/Either"));
const O = __importStar(require("fp-ts/Option"));
const A = __importStar(require("fp-ts/ReadonlyArray"));
const TE = __importStar(require("fp-ts/TaskEither"));
const function_1 = require("fp-ts/function");
const PathReporter = __importStar(require("io-ts/lib/PathReporter"));
const lerna_packages_1 = require("./lerna-packages");
const debug = {
    cmd: (0, debug_1.default)('link'),
};
const findPackageIn = (packages) => (packagePathOrName) => (0, function_1.pipe)(O.fromNullable(packages.get(packagePathOrName)), E.fromOption(() => ({
    type: 'unknown package',
    package: packagePathOrName,
})), TE.fromEither, TE.map((0, trace_1.trace)(debug.cmd, 'Linking dependencies for package')));
const packageManifest = (0, function_1.flow)(package_manifests_1.packageManifest, TE.map(({ contents }) => contents));
const decodeManifest = (manifest) => (0, function_1.pipe)(io_ts_1.PackageJsonDependencies.decode(manifest), E.mapLeft((0, function_1.flow)((errors) => PathReporter.failure(errors).join('\n'), (error) => ({ type: 'unable to decode package manifest', error }))), TE.fromEither);
const internalDependencies = (packages) => (dependencies) => {
    return (0, function_1.pipe)(dependencies, A.chain((dependency) => (0, function_1.pipe)(O.fromNullable(packages.get(dependency)), O.map(A.of), O.getOrElseW(() => A.empty))));
};
const internalPackageDependencies = (packages) => (0, function_1.flow)(packageManifest, TE.chainW(decodeManifest), TE.map((0, function_1.flow)(io_ts_1.dependencies, A.map(([dependency]) => dependency), internalDependencies(packages))));
const mkdir = (target) => (0, function_1.pipe)(TE.tryCatch(() => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve, reject) => {
        fs.mkdir(target, { recursive: true }, (error) => {
            if (error !== null && error !== undefined) {
                reject(error);
            }
            else {
                resolve((0, function_1.constVoid)());
            }
        });
    });
}), (0, function_1.flow)(E.toError, (error) => ({ type: 'unable to create directory', path: target, error }))));
/**
 * Note: defaults to no action when `target` already exists.
 * Wonder if I should instead remove the existing file before creating `target`.
 */
const symlink = (target, link) => (0, function_1.pipe)(mkdir(path.dirname(link)), TE.chainW(() => (0, function_1.pipe)(TE.tryCatch(() => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve, reject) => {
        fs.symlink(target, link, (error) => {
            if (error !== null && error !== undefined) {
                reject(error);
            }
            else {
                resolve((0, function_1.constVoid)());
            }
        });
    });
}), E.toError), 
// recover from symlink-already-exists error
TE.orElse((error) => error.message.startsWith('EEXIST:') ? TE.right((0, function_1.constVoid)()) : TE.left(error)), TE.mapLeft((error) => ({
    type: 'unable to create symlink',
    target,
    path: link,
    error,
})))));
const nodeModules = (packageDirectory) => path.resolve(packageDirectory, 'node_modules');
const symlinkPackage = (targetPackage) => (lernaPackage) => {
    const symlinkLocation = path.resolve(nodeModules(targetPackage), lernaPackage.name);
    debug.cmd('Linking ', path.relative(path.dirname(symlinkLocation), lernaPackage.location), 'to', symlinkLocation);
    return symlink(path.relative(path.dirname(symlinkLocation), lernaPackage.location), symlinkLocation);
};
const linkLocalDependenciesIn = (packages) => (packagePathOrName) => (0, function_1.pipe)(TE.Do, TE.bind('lernaPackage', () => findPackageIn(packages)(packagePathOrName)), TE.bindW('dependencies', ({ lernaPackage }) => internalPackageDependencies(packages)(lernaPackage)), TE.chainW(({ lernaPackage, dependencies }) => TE.traverseArray(symlinkPackage(lernaPackage.location))(dependencies)), TE.map(function_1.constVoid));
exports.linkLocalDependenciesIn = linkLocalDependenciesIn;
// REFACTOR: this is the result of a gnarly, time-constrained session
// It's not pretty code, I think we should remove the ./lerna-packages
// file entirely and consolidate our exposed api
const linkLocalDependencies = (packagePathOrName) => (0, function_1.pipe)(
// FIXME: process.cwd should instead be exposed as a parameter
(0, lerna_packages_1.lernaPackages)(process.cwd()), TE.chain((packages) => (0, exports.linkLocalDependenciesIn)(packages.map)(packagePathOrName)));
exports.linkLocalDependencies = linkLocalDependencies;
const linkAllLocalDependencies = (packagesMap, packagesList) => (0, function_1.pipe)(packagesList, TE.traverseArray((0, function_1.flow)((pkg) => pkg.location, (0, exports.linkLocalDependenciesIn)(packagesMap))), TE.map(function_1.constVoid));
exports.linkAllLocalDependencies = linkAllLocalDependencies;
//# sourceMappingURL=index.js.map